(* ::Package:: *)

(************************************************************************)
(* This file was generated automatically by the Mathematica front end.  *)
(* It contains Initialization cells from a Notebook file, which         *)
(* typically will have the same name as this file except ending in      *)
(* ".nb" instead of ".m".                                               *)
(*                                                                      *)
(* This file is intended to be loaded into the Mathematica kernel using *)
(* the package loading commands Get or Needs.  Doing so is equivalent   *)
(* to using the Evaluate Initialization Cells menu command in the front *)
(* end.                                                                 *)
(*                                                                      *)
(* DO NOT EDIT THIS FILE.  This entire file is regenerated              *)
(* automatically each time the parent Notebook file is saved in the     *)
(* Mathematica front end.  Any changes you make to this file will be    *)
(* overwritten.                                                         *)
(************************************************************************)



(* :Title: ConvexCones *)


(* :Context: "Convex`" *)


(* :Author: Christopher J. Herich
            19 Tillinghast Place
		 Buffalo NY 14216 *)


(* :Summary: Functions for constructing convex cones of
          arbitrary dimension, manipulating them, 
             and extracting their attributes.*)


(* :Copyright: Christopher J. Henrich 2007 *)


(* :Package version: 1.0.2 *)


(* :Mathematica Version: 6.0 *)


(* :History: Developed from Guenter M. Ziegler, Lectures on Polytopes, 
             Springer-Verlag 1995; 
             and the Parma Polyhedral Library
		   (http://www.cs.unipr.it/ppl).
          1.0.1: June 2009. Fixed bug in ProjectConeAlongAxis
							 and IntersectConeWithHyperplane 
		  1.0.2: December 12 2009. Fixed bug in AddDescriptor *)


(* :Keywords: convex polyhedra, convex cones, linear algebra,  
              multi-dimensional geometry*)


(* :Requirements:  *)


BeginPackage["Convex`ConvexCones`",{"Global`"}];


ConvexCone::usage =
"ConvexCone is the head of an expression denoting a closed convex polynomial cone.";


Hyperplanes::usage =
"Hyperplanes[cone] returns the list of vectors specifying hyperplanes which constrain cone.";


Halfspaces::usage =
"Halfspaces[cone] returns the list of vectors specifying halfspaces which constrain code.";


Lines::usage =
"Lines[cone] returns the list of vectors specifying line generators of cone.";


Rays::usage =
"Rays[cone] returns the list of vectors specifying ray generators of cone.";


AmbientDimension::usage =
"AmbientDimension[cone] is the dimension of the space in which cone resides.";


FullSpace::usage=
"FullSpace[d] constructs a cone which occupies the entire d-dimensional vector space.";


ZeroCone::usage =
"ZeroCone[d] constructs a cone consisting of the origin in d-dimensional vector space.";


ConeIntersection::usage =
"ConeIntersection[hyperplanes, halfspaces] constructs a cone with the given constraints.";


ConicalHull::usage=
"ConicalHull[lines, rays] constructs a cone with the given generators.";


AddHyperplaneToCone::usage=
"AddHyperplaneToCone[cone, constraint] constructs from the given cone a new one, adding the given hyperplane constraint.";


AddHalfspaceToCone::usage=
"AddHalfspaceToCone[cone, constraint] constructs from the given cone a new one, adding the given halfspace constraint.";


AddConstraintGroupToCone::usage =
"AddConstraintGroupToCone[cone, hyperplanes, halfspaces] constructs a new cone from the old one, adding new constraints.";


AddLineToCone::usage=
"AddLineToCone[cone, generator] constructs a new cone from the given one, adding the given line generator.";


AddRayToCone::usage =
"AddRayToCone[cone, generator] constructs a new cone from the given one, adding the given ray generator.";


AddGeneratorGroupToCone::usage =
"AddGeneratorGroupToCone[cone, lines, rays] constructs from the given cone a new one, adding the given generators.";


TransformGeneratorsOfCone::usage=
"TransformGeneratorsOfCone[cone, transformer] applies a linear transformation to the generators of a given cone, to obtain a new one.";


ProjectConeAlongAxis::usage =
"ProjectConeAlongAxis[cone, axis] returns the projection of the given cone along the given axis.";


IntersectConeWithHyperplane::usage =
"IntersectConeWithHyperplane[cone, axis] returns the intersection of the given cone with the coordinate hyperplane orthogonal to the given axis.";


ConeContains::usage=
"ConeContains[cone, point] assumes that cone is a convexCone and point is a vector of the same dimension as the descriptors of cone.  It returns True if & only if point is contained in conef.";


SubCone::usage=
"SubCone[cone1,cone2]assumes that the arguments are ConvexCones of equal dimension.  It returns True if and only if cone1 is a subset of cone2.";


EqualCone::usage=
"EqualCone[cone1,cone2] assumes that cone1 and cone2 are ConvexCone's of equal dimension.It returns True if & only if they are equal as point sets.";


Begin["`Private`"];


Hyperplanes[ConvexCone[d_,hp_,hs_,ln_, ra_]] := hp;


Halfspaces[ConvexCone[d_,hp_,hs_,ln_,ra_]] := hs;


Lines[ConvexCone[d_,hp_,hs_,ln_,ra_]] := ln;


Rays[ConvexCone[d_,hp_,hs_,ln_,ra_]]:= ra;


AmbientDimension[ConvexCone[d_,hp_,hs_,ln_,ra_]] := d;


FullSpace[d_] :=ConvexCone[d,{},{},IdentityMatrix[d],{}];


ZeroCone[d_] := ConvexCone[d,IdentityMatrix[d],{},{},{}];


ConeIntersection[hp_, hs_] := Catch[
Module[{d,rawCone,newHp,newHs,newLn, newRa},
d=DimVet[hp,hs];
rawCone=FullSpace[d];
{{newHp,newHs},{newLn,newRa}} = AddDescriptions[d, {{Hyperplanes@rawCone,Halfspaces@rawCone},{Lines@rawCone,Rays@rawCone}},{hp,hs}];
SimplifyCone[ConvexCone[d, newHp, newHs, newLn, newRa]]
],
notLists|emptyArgs|notUniform,
FullSpace[#1]&
];


ConicalHull[ln_,ra_] := Catch[
Module[{d,rawCone,newHp,newHs,newLn, newRa},
d=DimVet[ln, ra];
rawCone=ZeroCone[d];
{{newLn,newRa},{newHp,newHs}} = AddDescriptions[d, {{Lines@rawCone,Rays@rawCone},{Hyperplanes@rawCone,Halfspaces@rawCone}},{ln,ra}];
SimplifyCone[ConvexCone[d, newHp, newHs, newLn, newRa]]
],
notLists|emptyArgs|notUniform,
ZeroCone[#1]&
];


isTwoSided = True;
isOneSided = False;


AddHyperplaneToCone[ConvexCone[d_,hp_,hs_,ln_,ra_], constraint_] :=
Catch[Module[{newHp,newHs,newLn,newRa},
DimVetOne[d,constraint];
{{newHp,newHs},{newLn,newRa}} =
AddDescriptor[d,{{hp,hs},{ln,ra}},constraint,isTwoSided];
SimplifyCone[ConvexCone[d, newHp, newHs, newLn, newRa]]
],
notLists|wrongDimension,
ConvexCone[d,hp,hs,ln,ra]
];


AddHalfspaceToCone[ConvexCone[d_,hp_,hs_,ln_,ra_], constraint_] :=
Catch[Module[{newHp,newHs,newLn,newRa},
DimVetOne[d,constraint];
{{newHp,newHs},{newLn,newRa}} =
AddDescriptor[d,{{hp,hs},{ln,ra}},constraint,isOneSided];
SimplifyCone[ConvexCone[d, newHp, newHs, newLn, newRa]]
],
notLists|wrongDimension,
ConvexCone[d,hp,hs,ln,ra]
];


AddConstraintGroupToCone[ConvexCone[d_,hp_,hs_,ln_,ra_],nhp_,nhs_] := Catch[
Module[{newHp,newHs,newLn, newRa},
DimVet[d,nhp,nhs];
{{newHp,newHs},{newLn,newRa}} = AddDescriptions[d, {{hp,hs},{ln,ra}},{nhp,nhs}];
SimplifyCone[ConvexCone[d, newHp, newHs, newLn, newRa]]
],
notLists| notUniform,
ConvexCone[d,hp,hs,ln,ra]
];



AddLineToCone[ConvexCone[d_,hp_,hs_,ln_,ra_], generator_]:= 
Catch[Module[{newHp,newHs,newLn,newRa},
DimVetOne[d, generator];
{{newLn,newRa},{newHp,newHs}} =
AddDescriptor[d,{{ln,ra},{hp,hs}},generator,isTwoSided];
SimplifyCone[ConvexCone[d, newHp, newHs, newLn, newRa]]
],
notLists|wrongDimension,
ConvexCone[d,hp,hs,ln,ra]
];


AddRayToCone[ConvexCone[d_,hp_,hs_,ln_,ra_], generator_]:= 
Catch[Module[{newHp,newHs,newLn,newRa},
DimVetOne[d, generator];
{{newLn,newRa},{newHp,newHs}} =
AddDescriptor[d,{{ln,ra},{hp,hs}},generator, isOneSided];
SimplifyCone[ConvexCone[d, newHp, newHs, newLn, newRa]]
],
notLists|wrongDimension,
ConvexCone[d,hp,hs,ln,ra]
];


AddGeneratorGroupToCone[ConvexCone[d_,hp_,hs_,ln_,ra_], nln_, nra_] := Catch[
Module[{newHp,newHs,newLn, newRa},
DimVet[d,nln,nra];
{{newLn,newRa},{newHp,newHs}} = AddDescriptions[d, {{ln,ra},{hp,hs}},{nln,nra}];
SimplifyCone[ConvexCone[d,newHp,newHs,newLn,newRa]]
],
notLists|notUniform,
ConvexCone[d,hp,hs,ln,ra]
];


TransformGeneratorsOfCone[ConvexCone[d_, hp_, hs_, ln_, ra_], transformer_] := Catch[
With[{contraTrans = Check[Inverse@Transpose@transformer,Throw[0,singular]]},
DimVet[d,transformer,{}];ConvexCone[d,(contraTrans.#1)&/@hp, (contraTrans.#1)&/@hs,(transformer.#1)&/@ln,(transformer.#1)&/@ra]
],
notLists|wrongDimension|singular,
ConvexCone[d, hp, hs, ln, ra]
];


ProjectConeAlongAxis[ConvexCone[d_, hp_, hs_, ln_, ra_], axis_] :=Module[{newLn,newRa,newHp,newHs,cutter,newCone,newDescriptor,minDim},
newDescriptor=UnitVector[d,axis];
minDim = d - Length[hp] - Length[ln] - 2;
{{newLn,newRa},{newHp,newHs}} = AddDescriptor[d,{{ln,ra},{hp,hs}},newDescriptor,isTwoSided];
cutter=Drop[#1,{axis}]&;
newCone =ConvexCone[d-1,cutter/@newHp, cutter/@newHs, cutter/@newLn, cutter/@newRa];
SimplifyCone[newCone]
];


IntersectConeWithHyperplane[ConvexCone[d_, hp_, hs_, ln_, ra_],axis_ ] :=
Module[{newLn,newRa,newHp,newHs,cutter,newCone,newDescriptor,minDim},
newDescriptor = UnitVector[d,axis];
minDim = d - Length[hp] - Length[ln] - 2;
{{newHp,newHs},{newLn,newRa}} =AddDescriptor[d,{{hp,hs},{ln,ra}},newDescriptor,isTwoSided];
cutter=Drop[#1,{axis}]&;
newCone =ConvexCone[d-1,cutter/@newHp, cutter/@newHs, cutter/@newLn, cutter/@newRa];
SimplifyCone[newCone]
];


ConeContains[ConvexCone[d_, hp_, hs_, ln_, ra_],point_] := (And@@((#1.point==0)&/@hp))&&(And@@((#1.point<=0)&/@hs));


SubCone[ConvexCone[d_, hp_, hs_, ln_, ra_],cone2_] := (And@@(ConeContains[cone2,#1]&/@ln))&&(And@@(ConeContains[cone2,-#1]&/@ln))&&(And@@(ConeContains[cone2,#1]&/@ra));


EqualCone[cone1_,cone2_] := SubCone[cone1,cone2]&&SubCone[cone2,cone1];


{notLists,emptyArgs,notUniform,wrongDimension,singular}; (*Tags*)


DimVet[{},{}]:=Throw[3,emptyArgs];
DimVet[arg1_List, arg2_List] := With[{allarg=Join[arg1,arg2]},
If[And@@(MatchQ[#1,_List]&/@allarg),
(* All members are lists. Should be of uniform length*)
With[{d=Length@First@allarg},
If[And@@((Length@#1 == d)&/@allarg),
(*all's well *)d,
Throw[d,notUniform]]],
(* Eheu! They aren't all Lists. *)
Throw[2,notLists]]
];
DimVet[arg1_,arg2_] :=Throw[3,notLists];


DimVet[d_,arg1_List,arg2_List] := With[{allarg=Join[arg1,arg2]},
If[And@@(MatchQ[#1,_List]&/@allarg),
(* All members are lists. Should be of length d.*)
If[And@@((Length@#1 == d)&/@allarg),
(*all's well *)Null,
Throw[Null,notUniform]],
(* They are not all of the right length *)
Throw[Null,notLists]]
];
DimVet[d_,arg1_,arg2_] := Throw[Null,notLists];


DimVetOne[d_, desc_List] := If[Length[desc]==d,
Null,
Throw[Null,wrongDimension]
];
DimVetOne[d_,desc_] := Throw[Null,notLists];


AddDescriptions[d_,descriptions_,{newTwoSide_,newOneSide_}] :=Fold[AddDescriptor[d,#1,#2,isOneSided]&,Fold[AddDescriptor[d,#1,#2, isTwoSided]&,
descriptions,newTwoSide],
newOneSide];


AddDescriptor[d_,{{primFull_,primHalf_}, {secondFull_, secondHalf_}}, newDescriptor_, twoSidedFlag_] := Module[{scalarProducts,ns,minDim,removable,newFull,newHalf},
(*look for a secondary descriptor that does not saturate newDescriptor*)
scalarProducts = (newDescriptor .#)& /@ Join[secondFull,secondHalf];
ns = Pick[Range[Length[scalarProducts]],
(#1 != 0)& /@ scalarProducts,True];
Which[
ns == {},
	(* All secondaries saturate. newDescriptor is redundant*)
{{primFull,primHalf},{secondFull,secondHalf}},

First[ns]<= Length[secondFull],
	(* A full secondary does not saturate *)
With[{fns=First@ns},
removable=secondFull[[fns]];
If[(!twoSidedFlag)&&(scalarProducts[[fns]]>0),removable=-removable];
newFull=ShiftDescriptors[newDescriptor,removable,Drop[secondFull, {fns}]];
newHalf=ShiftDescriptors[newDescriptor, removable,secondHalf];
If[twoSidedFlag,
{{Append[primFull,newDescriptor],primHalf},{newFull,newHalf}},
(*else*)
{{primFull,Append[primHalf,newDescriptor]},{newFull,Append[newHalf,removable]}}
]
],

True,
	(* All full secondaries saturate, but not all Half s. *)
minDim = d - Length[primFull]-Length[secondFull]-2;
newHalf=FME[primHalf,secondHalf,newDescriptor,minDim];
If[twoSidedFlag,
{{Append[primFull,newDescriptor],primHalf},{secondFull,newHalf}},
(*else*)
{{primFull,Append[primHalf,newDescriptor]},{secondFull,Join[Select[secondHalf,(#1.newDescriptor<0)&],newHalf]}}

]
]
];


FME[primaryHalf_, secondaryHalf_, newPrimary_,minDim_] :=Module[{combiner, adjacentQ,adjacentPairs, ups, downs,satLists},
ups=SelectIndex[secondaryHalf, (#1.newPrimary > 0)&];
downs=SelectIndex[secondaryHalf,(#1.newPrimary <0)&];
satLists = Table[SaturationSet[secondaryHalf[[i]],primaryHalf],{i,Length@secondaryHalf}];

(*predicate of indexes of two descriptors*)
adjacentQ[i_,j_] := With[{sigmaIJ = Intersection[satLists[[i]],satLists[[j]]]},(Length[sigmaIJ]>=minDim)&&And@@Table[(k==i)||(k==j)||(Length[Complement[sigmaIJ,satLists[[k]]]]>0),{k,Length[secondaryHalf]}]];

(* function used in making new descriptors*)
combiner=Regularize[(secondaryHalf[[#1]].newPrimary)secondaryHalf[[#2]]-(secondaryHalf[[#2]].newPrimary)secondaryHalf[[#1]]]&;

adjacentPairs = Select[Flatten[Outer[List,ups,downs],1],(adjacentQ@@#1)&];
Join[Select[secondaryHalf, (#1.newPrimary == 0)&],(combiner@@#1)& /@ adjacentPairs]
];


SimplifyCone[cone_] := cone // FirstOptimize // SecondOptimize // ThirdOptimize // FullOptimize;


FirstOptimize[ConvexCone[d_,hp_,hs_,ln_,ra_]] := Module[{satRays, unSatRays, satHs, unSatHs},satRays = SelectIndex[ra,(Length[hs] == Length@SaturationSet[#1,hs])&];unSatRays = Complement[Range@Length@ra,satRays];
satHs = SelectIndex[hs,(Length[ra] == Length@SaturationSet[#1,ra])&]; unSatHs = Complement[Range@Length@hs,satHs];
ConvexCone[d,hp ~ Join ~hs[[satHs]],hs[[unSatHs]],ln ~ Join ~ ra[[satRays]],ra[[unSatRays]]]
];


SecondOptimize[ConvexCone[d_,hp_,hs_,ln_,ra_]] := Module[{newHp,newHs,newLn,newRa},{newHp,newHs}=Fold[EliminateByRow,{hp,hs},Range@Length@hp];{newLn,newRa}=Fold[EliminateByRow,{ln,ra},Range@Length@ln];ConvexCone[d,Select[newHp,IsNotZeroQ],Select[newHs,IsNotZeroQ],Select[newLn,IsNotZeroQ],Select[newRa,IsNotZeroQ]]
];


EliminateByRow[{fullDesc_,halfDesc_},i_] := With[{jl=Select[Range@Length[fullDesc[[i]]],(0 != fullDesc[[i,#1]])&,1]},
EBR2[{fullDesc,halfDesc},i,jl]
];


EBR2[description_,i_,{}]:=description;


EBR2[{fullDesc_,halfDesc_},i_,{j_}] :=Module[{fullA,fullB,gauss,den,row},fullA=Take[fullDesc,i-1]; fullB=Take[fullDesc,{i+1,Length@fullDesc}];
row=fullDesc[[i]];
den=row[[j]];
gauss=(#1 - (#1[[j]]/den)*row)&;
{Join[gauss/@fullA,{row},gauss/@fullB],gauss/@halfDesc}
];


IsNotZeroQ[desc_] :=Length[Select[desc,(0!= #1)&]] >0;


ThirdOptimize[ConvexCone[d_,hp_,hs_,ln_,ra_]] := ConvexCone[d,hp,WeedOut@hs,ln,WeedOut@ra];


WeedOut[dlist_] := Module[{n,pairs,props,drops},
n=Length@dlist;pairs=Flatten[Table[Table[{i,j},{j,i+1,n}],{i,1,n}],1];props=Select[pairs,(ProportionalQ@@dlist[[#1]])&];drops=Union[Last/@props];
dlist[[Complement[Range[n],drops]]]
];


ProportionalQ[desc1_,desc2_] := ((desc1.desc2)^2 == (desc1.desc1)(desc2.desc2));


FullOptimize[ConvexCone[d_,hp_,hs_,ln_,ra_]] := Module[{newHs,newRa,minDim},
minDim = d - Length[hp]-Length[ln]-1;newHs =OptimizeDescList[hs,ra,minDim]; newRa=OptimizeDescList[ra,hs,minDim];
ConvexCone[d,hp,newHs,ln,newRa]
];


OptimizeDescList[primaryDesc_,secondaryDesc_,minDim_]:=Module[{satLists,essentialIndexes},satLists = Table[SaturationSet[primaryDesc[[i]],secondaryDesc],{i,Length@primaryDesc}];essentialIndexes=Select[Range[Length[primaryDesc]],((Length[satLists[[#1]]]>= minDim)&&(And@@Table[(k==#1)||Length[Complement[satLists[[#1]],satLists[[k]]]]>0,{k,Length@primaryDesc}]))&];
primaryDesc[[essentialIndexes]]
];


SaturationSet[primeDesc_,secondDescList_] := SelectIndex[secondDescList,(primeDesc.#1 == 0)&];


SelectIndex[list_,crit_] := Select[Range@Length@list,(crit[list[[#1]]])&];


ShiftDescriptors[normal_, addend_, descriptorList_] :=Regularize /@(descriptorList - (1/(normal.addend))KroneckerProduct[(#1.normal)&/@descriptorList,addend]);


Regularize[vector_] := With[{nz=Select[vector,(# != 0)&]},
If[Length[nz]==0,
vector, (*else*)
(1/Abs[Last[nz]])vector]];


End[];


EndPackage[];
