(* ::Package:: *)

(************************************************************************)
(* This file was generated automatically by the Mathematica front end.  *)
(* It contains Initialization cells from a Notebook file, which         *)
(* typically will have the same name as this file except ending in      *)
(* ".nb" instead of ".m".                                               *)
(*                                                                      *)
(* This file is intended to be loaded into the Mathematica kernel using *)
(* the package loading commands Get or Needs.  Doing so is equivalent   *)
(* to using the Evaluate Initialization Cells menu command in the front *)
(* end.                                                                 *)
(*                                                                      *)
(* DO NOT EDIT THIS FILE.  This entire file is regenerated              *)
(* automatically each time the parent Notebook file is saved in the     *)
(* Mathematica front end.  Any changes you make to this file will be    *)
(* overwritten.                                                         *)
(************************************************************************)



(* :Title: ConvexPolyhedra *)


(* :Context: "Convex`" *)


(* :Author: Christopher J. Herich
            19 Tillinghast Place
		 Buffalo NY 14216 *)


(* :Summary: Functions for constructing convex polyhedra of
          arbitrary dimension, manipulating them, 
             and extracting their attributes.*)


(* :Copyright: Christopher J. Henrich 2007 *)


(* :Package version: 1.0.1 *)


(* :Mathematica Version: 6.0 *)


(* :History: Developed from Guenter M. Ziegler, Lectures on Polytopes, 
             Springer-Verlag 1995; 
             and the Parma Polyhedral Library
		   (http://www.cs.unipr.it/ppl).
          1.0.1: June 2009. Minor changes.*)


(* :Keywords: convex polyhedra, convex cones, linear algebra,  
              multi-dimensional geometry*)


(* :Requirements: Convex`ConvexCones.nb *)


BeginPackage["Convex`ConvexPolyhedra`",{"Global`","Convex`ConvexCones`"}];


ConvexPolyhedron::usage=
"ConvexPolyhedron is the head of an expression denoting a closed convex polyhedron.";


Points::usage=
"Points[polyhedron] returns the list of vectors specifying point generators of the polyhedron.";


Rays::usage=
"Rays[polyhedron] returns the list of vectors specifying ray generators of the polyhedron.";


Lines::usage=
"Lines[polyhedron] returns the list of vectors specifying line generators of the polyhedron.";


Halfspaces::usage =
"Halfspaces[polyhedron] returns the list of descriptors specifying halfspace constraints on the polyhedron.";


Hyperplanes::usage=
"Hyperplanes[polyhedron] returns the list of descriptors specifying hyperplane constraints on the polyhedron.";


AmbientDimension::usage=
"AmbientDimension[polyhedron] returns the dimension of the vector space in which polyhedron is situated.";


LinealityDimension::usage=
"LinealityDimension[polyhedron] returns the dimension of the lineality space of the polyhedron; this is the dimension of a maximal affine subspace that is contained in the polyhedron.";


SpanDimension::usage=
"SpanDimension[polyhedron] returns the dimension of the smallest affine subspace containing the polyhedron.";


FullSpacePolyhedron::usage=
"FullSpacePolyhedron[d] returns a ConvexPolyhedron occupying the entire d-dimensional vector space.";


EmptyPolyhedron::usage=
"emptyPolyhedron[d] returns a ConvexPolyhedron which is empty, understood to be contained in d-dimensional vector space.";


PolyhedronIntersection::usage=
"PolyhedronIntersection[hyperplanes, halfspaces] constructs a ConvexPolyhedron defined by the given constraints.";


PolyhedronHull::usage=
"PolyhedronHull[lines, rays, points] constructs a ConvexPolyhedron defined by the given generators.";


AddPointToPolyhedron::usage=
"AddPointToPolyhedron[polyhedron, generator] interprets 'generator' as a point, and constructs a new ConvexPolyhedron from the old one by adding the new point.";


AddRayToPolyhedron::usage=
"AddRayToPolyhedron[polyhedron, generator] interprets 'generator' as a ray, and constructs a new ConvexPolyhedron from the given one by adding the new ray.";


AddLineToPolyhedron::usage=
"AddLineToPolyhedron[polyhedron, genrator] interprets 'generator' as a line, and constructs a new ConvexPolyhedron from the given one by ading the new line.";


AddHalfspaceToPolyhedron::usage=
"AddHalfspaceToPolyhedron[polyhedron, constraint] interprets 'constraint' as the descriptor of a halfspace, and returns a new ConvexPolyhedron which represents the intersection of the given polyhedron with the halfspace.";


AddHyperplaneToPolyhedron::usage=
"AddHyperplaneToPolyhedron[polyhedron, constraint' interprets 'constraint' as the descriptor of a hyperplane, and returns a new ConvexPolyhedron which represents the intersection of the given polyhedron with the hyperplane.";


AddGeneratorsToPolyhedron::usage=
"AddGeneratorsToPolyhedron[polyhedron, lines, rays, points] returns a new ConvexPolyhedron constructed from the given oneby adding the generators listed as 'lines,' 'rays,' and 'points.'";


AddConstraintsToPolyhedron::usage=
"AddConstraintsToPolyhedron[polyhedron, hyperplanes, halfspaces] returns a new ConvexPolyhedron constructed from the given one by imposing the constraints listed as 'halfspaces' and 'hyperplanes.'";


TransformGeneratorsOfPolyhedron::usage=
"TransformGeneratorsOfPolyhedron[polyhedron, transform, translation] transforms the generators of 'polyhedron' to return a new ConvexPolyhedron.  The line and ray generators are acted on by the linear transformation 'transform'. The point generators are acted on by an affine transformation - 'transform' followed by the adding of 'translation'.";


ProjectPolyhedronAlongAxis::usage=
"ProjectPolyhedronAlongAxis[polyhedron,axis] returns a new ConvexPolyhedron constructed from the given one by projecting along the given 'axis.' The component indexed by 'axis' is dropped from the descriptors.";


IntersectPolyhedronWithHyperplane::usage=
"IntersectPolyhedronWithHyperplane[polyhedron, axis] constructs a new ConvexPolyhedron from the given one by intersecting it with the hyperplane on which the component of index 'axis' is 0. This component is dropped from the descriptors.";


PolyhedronContainsPoint::usage=
"PolyhedronContainsPoint[polyhedron, point] returns True iff the given 'point' is contained in the given ConvexPolyhedron.";


PolyhedronContainsRay::usage=
"PolyhedronContainsRay[polyhedron,ray] returns True iff a ray with the given direction, issuing from an arbitrary point in the given ConvexPolyhedron, is contained in it.";


SubPolyhedron::usage=
"SubPolyhedron[polyhedron1, polyhedron2] compares two convex polyhedra as point sets, and returns True iff the first is a subset of the second.";


EqualPolyhedron::usage=
"EqualPolyhedron[polyhedron1,polyhedron2] compares to convex polyhedra as point sets, and returns True iff they are equal.";


Begin["`Private`"];


Points[ConvexPolyhedron[k_]] := With[{coneRays=Rays@k},With[{finiteRays=Select[coneRays,(Last[#1]>0)&]},ToLowerSpace/@finiteRays]];


Rays[ConvexPolyhedron[k_]] := With[{coneRays=Rays@k},With[{infiniteRays=Select[coneRays,(Last[#1]==0)&]},Truncate/@infiniteRays]];


Lines[ConvexPolyhedron[k_]] := With[{coneLines = Lines@k},(Drop[#1,-1]&)/@coneLines];


Halfspaces[ConvexPolyhedron[k_]] := Halfspaces[k];


Hyperplanes[ConvexPolyhedron[k_]] := Hyperplanes[k];


AmbientDimension[ConvexPolyhedron[k_]] := AmbientDimension[k]-1;


LinealityDimension[ConvexPolyhedron[k_]] := Length@Lines@k;


SpanDimension[ConvexPolyhedron[k_]] := AmbientDimension@k-Length@Hyperplanes@k-1;


FullSpacePolyhedron[d_] := ConvexPolyhedron[ConeIntersection[{},{-UnitVector[d+1,d+1]}]];


EmptyPolyhedron[d_] := ConvexPolyhedron[ZeroCone[d+1]];


PolyhedronIntersection[hyperplanes_,halfspaces_] := Catch[
With[{d1=DimVet[hyperplanes,halfspaces]},
ConvexPolyhedron[ConeIntersection[hyperplanes,Join[halfspaces,{-UnitVector[d1,d1]}]]]],
notLists | emptyArgs | notUniform,
FullSpacePolyhedron[#1-1]&
];


PolyhedronHull[lines_,rays_,points_]:= Catch[
With[{d=DimVet[{},points]},
DimVet[d,lines,rays];
ConvexPolyhedron[ConicalHull[ToUpperSpace0/@lines,Join[ToUpperSpace0/@rays,ToUpperSpace/@points]]]
],
notLists|emptyArgs|notUniform|wrongDimension,
EmptyPolyhedron[#1]&
];


AddPointToPolyhedron[ConvexPolyhedron[cone_],generator_]:=ConvexPolyhedron[AddRayToCone[cone,ToUpperSpace@generator]];


AddRayToPolyhedron[ConvexPolyhedron[cone_],generator_]:=ConvexPolyhedron[AddRayToCone[cone,ToUpperSpace0@generator]];


AddLineToPolyhedron[ConvexPolyhedron[cone_],generator_]:=ConvexPolyhedron[AddLineToCone[cone,ToUpperSpace0@generator]];


AddHalfspaceToPolyhedron[ConvexPolyhedron[cone_],generator_]:=ConvexPolyhedron[AddHalfspaceToCone[cone,generator]];


AddHyperplaneToPolyhedron[ConvexPolyhedron[cone_],generator_]:=ConvexPolyhedron[AddHyperplaneToCone[cone,generator]];


AddGeneratorsToPolyhedron[ConvexPolyhedron[cone_],lines_,rays_,points_]:=ConvexPolyhedron[AddGeneratorGroupToCone[cone,ToUpperSpace0/@lines,Join[ToUpperSpace0/@rays,ToUpperSpace/@points]]];


AddConstraintsToPolyhedron[ConvexPolyhedron[cone_],hyperplanes_,halfspaces_]:=  ConvexPolyhedron[AddConstraintGroupToCone[cone,hyperplanes,halfspaces]];


TransformGeneratorsOfPolyhedron[ConvexPolyhedron[cone_],transform_,translation_]:=
With[{uT1=ToUpperSpace0/@(Transpose@transform)},
With[{uT0=Join[uT1,{ToUpperSpace@translation}]},With[{upperTrans=Transpose@uT0},ConvexPolyhedron[TransformGeneratorsOfCone[cone,upperTrans]]
]]];


ProjectPolyhedronAlongAxis[ConvexPolyhedron[cone_],axis_] :=ConvexPolyhedron[ProjectConeAlongAxis[cone,axis]];


IntersectPolyhedronWithHyperplane[ConvexPolyhedron[cone_],axis_]:= ConvexPolyhedron[IntersectConeWithHyperplane[cone,axis]];


PolyhedronContainsPoint[ConvexPolyhedron[cone_],point_]:=ConeContains[cone,ToUpperSpace@point];


PolyhedronContainsRay[ConvexPolyhedron[cone_],ray_]:= ConeContains[cone,ToUpperSpace0@ray];


SubPolyhedron[ConvexPolyhedron[cone1_],ConvexPolyhedron[cone2_]]:=SubCone[cone1,cone2];


EqualPolyhedron[ConvexPolyhedron[cone1_],ConvexPolyhedron[cone2_]]:=EqualCone[cone1,cone2];


{notLists,emptyArgs,notUniform,wrongDimension,singular}; (*Tags*)


DimVet[{},{}]:=Throw[3,emptyArgs];
DimVet[arg1_List, arg2_List] := With[{allarg=Join[arg1,arg2]},
If[And@@(MatchQ[#1,_List]&/@allarg),
(* All members are lists. Should be of uniform length*)
With[{d=Length@First@allarg},
If[And@@((Length@#1 == d)&/@allarg),
(*all's well *)d,
Throw[d,notUniform]]],
(* Eheu! They aren't all Lists. *)
Throw[2,notLists]]
];
DimVet[arg1_,arg2_] :=Throw[3,notLists];


DimVet[d_,arg1_List,arg2_List] := With[{allarg=Join[arg1,arg2]},
If[And@@(MatchQ[#1,_List]&/@allarg),
(* All members are lists. Should be of length d.*)
If[And@@((Length@#1 == d)&/@allarg),
(*all's well *)Null,
Throw[Null,notUniform]],
(* They are not all of the right length *)
Throw[Null,notLists]]
];
DimVet[d_,arg1_,arg2_] := Throw[Null,notLists];


DimVetOne[d_, desc_List] := If[Length[desc]==d,
Null,
Throw[Null,wrongDimension]
];
DimVetOne[d_,desc_] := Throw[Null,notLists];


ToLowerSpace[desc_] := Drop[desc,-1]/Last[desc];


Truncate[descriptor_] := Drop[descriptor,-1];


ToUpperSpace[descriptor_] := Append[descriptor,1];


ToUpperSpace0[descriptor_] := Append[descriptor,0];


End[];


EndPackage[];
